@@ -1,7 +1,7 @@
 /*
  * gtk_anyone.c: Show the user tree as a Gtk+ widget
  * David Maze <dmaze@mit.edu>
- * $Id: gtkanyone.c,v 1.13 2000/04/20 18:11:14 dmaze Exp $
+ * $Id: gtkanyone.c,v 1.15 2001/10/20 14:47:02 dmaze Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,6 +15,9 @@
 static void gtk_anyone_class_init(GtkAnyoneClass *klass);
 static void gtk_anyone_init(GtkAnyone *anyone);
 
+static void rebuild_children(GtkAnyone *anyone);
+static void set_tooltip(GtkAnyone *anyone, GtkWidget *child, zuser *user);
+
 /* Populating the object
  * 
  * build_children() is called by gtk_anyone_construct(); it adds every
@@ -40,14 +43,7 @@ static void update_iter(zuser *user, gpointer data);
 
 static void gtk_anyone_class_init(GtkAnyoneClass *klass)
 {
-  GtkWidget *item;
-  
-  /* Create the popup menu (once). */
-  klass->popup_menu = gtk_menu_new();
-  
-  /* Create the update item and add it. */
-  item = gtk_menu_item_new_with_label("Update");
-  gtk_menu_append(GTK_MENU(klass->popup_menu), item);
+  /* Do nothing for now; see the Gtk+ tutorial if we ever add signals. */
 }
 
 static void gtk_anyone_init(GtkAnyone *anyone)
@@ -137,25 +133,6 @@ static void build_children(GtkAnyone *anyone)
 
   g_return_if_fail(anyone != NULL);
 
-  /* Add all of the group's users. */
-  base = zgroup_users(anyone->group);
-  for (list = base; list; list = g_list_next(list))
-  {
-    /* Get the child user object... */
-    user = list->data;
-    
-    /* Build a GtkTreeItem for it... */
-    name = zuser_name(user);
-    child = gtk_tree_item_new_with_label(name->str);
-    g_string_free(name, TRUE);
-
-    /* Set its user data to the relevant zuser... */
-    gtk_object_set_user_data(GTK_OBJECT(child), user);
-    
-    /* Add the tree item to the tree. */
-    gtk_tree_append(GTK_TREE(anyone), child);
-  }
-
   /* Recursively add all of the group's child groups. */
   base = zgroup_groups(anyone->group);
   for (list = base; list; list = g_list_next(list))
@@ -216,6 +193,8 @@ void gtk_anyone_update(GtkAnyone *anyone, GtkProgress *progress)
   uidata.count = 0;
   uidata.total = zusertable_count(table);
   zusertable_foreach(table, update_iter, &uidata);
+
+  rebuild_children(anyone);
 }
 
 static void update_iter(zuser *user, gpointer data)
@@ -225,8 +204,7 @@ static void update_iter(zuser *user, gpointer data)
   /* Figure out where they are. */
   zuser_update(user);
 
-  /* Show or hide the user widget as appropriate. */
-  gtk_anyone_show_or_hide_user(uidata->anyone, user);
+  /* To consider: incremental updates of the user tree? */
 
   /* Update the progress bar. */
   uidata->count++;
@@ -251,61 +229,122 @@ void gtk_anyone_update_user(GtkAnyone *anyone, zuser *user)
 
   /* Update the user and show them. */
   zuser_update(user);
-  gtk_anyone_show_or_hide_user(anyone, user);
+  rebuild_children(anyone);
 }
 
-void gtk_anyone_show_or_hide_user(GtkAnyone *anyone, zuser *user)
+static void rebuild_children(GtkAnyone *anyone)
 {
-  if (zuser_count_locs(user) > 0)
-    gtk_anyone_show_user(anyone, user);
-  else
-    gtk_anyone_hide_user(anyone, user);
-}
+  /* The plan: iterate simultaneously through the list of Gtk+ widget
+   * children and the znola list of users.  When there are discrepancies,
+   * patch up the widget tree to match.  There gets to be at most one
+   * child widget per user.  While we're at it, make sure that all of
+   * the child widgets are visible, and update the tooltips on all of
+   * them. */
+  GList *luser, *lwidget;
+  int pos = 0;
+  luser = zgroup_users(anyone->group);
+  lwidget = gtk_container_children(GTK_CONTAINER(anyone));
 
-void gtk_anyone_show_user(GtkAnyone *anyone, zuser *user)
-{
-  GList *base, *list;
-  
-  /* Use a callback to conditionally show each user. */
-  gtk_container_foreach(GTK_CONTAINER(anyone),
-			(GtkCallback)show_user_callback,
-			user);
+  while (lwidget)
+  {
+    /* Assume that things aren't totally messed up, and that lwidget's
+     * object data points to a valid user.  Walk through the list of
+     * users until we find that one, inserting things as we go. */
+    GtkWidget *widget = GTK_WIDGET(lwidget->data);
+    zuser *target, *user;
+
+    /* Sanity check: the widget should be a GtkTreeItem.  Stop if it
+     * has a subtree (since then we'll have reached the end of our own
+     * users). */
+    if (GTK_TREE_ITEM_SUBTREE(widget))
+      break;
+
+    target = (zuser *)gtk_object_get_user_data(GTK_OBJECT(widget));
+    user = luser->data;
+
+    while (user != target)
+    {
+      /* If the user is visible, create a widget for them. */
+      if (zuser_count_locs(user) > 0)
+      {
+        GtkWidget *child;
+        GString *name;
+
+        name = zuser_name(user);
+        child = gtk_tree_item_new_with_label(name->str);
+        g_string_free(name, TRUE);
+
+        /* Set its user data to the relevant zuser. */
+        gtk_object_set_user_data(GTK_OBJECT(child), user);
+    
+        /* Add the tree item to the tree. */
+        gtk_tree_insert(GTK_TREE(anyone), child, pos++);
+        gtk_widget_show(child);
+        set_tooltip(anyone, child, user);
+      }
+      
+      luser = luser->next;
+      user = luser->data;
+    }
+    /* Now we have a guarantee that user == target. */
+
+    /* Advance to the next widget now, before we possibly destroy the
+     * current item. */
+    lwidget = lwidget->next;
+    luser = luser->next;
+    pos++;
+
+    /* Check that the user is visible... */
+    if (zuser_count_locs(user) > 0)
+    {
+      set_tooltip(anyone, widget, user);
+    }
+    else
+    {
+      /* Nope, not around.  Delete their widget item. */
+      gtk_container_remove(GTK_CONTAINER(anyone), widget);
+    }
+  }
+
+  /* At this point, we've gone through all of the user widgets that
+   * exist.  If there are still users left, process and add them. */
+  while (luser)
+    {
+      zuser *user = luser->data;
+      
+      /* If the user is visible, create a widget for them. */
+      if (zuser_count_locs(user) > 0)
+      {
+        GtkWidget *child;
+        GString *name;
+        
+        name = zuser_name(user);
+        child = gtk_tree_item_new_with_label(name->str);
+        g_string_free(name, TRUE);
+        
+        /* Set its user data to the relevant zuser. */
+        gtk_object_set_user_data(GTK_OBJECT(child), user);
+        
+        /* Add the tree item to the tree. */
+        gtk_tree_insert(GTK_TREE(anyone), child, pos++);
+        gtk_widget_show(child);
+        set_tooltip(anyone, child, user);
+      }
+      
+      luser = luser->next;
+    }
 
   /* Recurse into subgroups. */
-  base = anyone->children;
-  for (list = base; list; list = g_list_next(list))
-    gtk_anyone_show_user(GTK_ANYONE(list->data), user);
+  for (lwidget = anyone->children; lwidget; lwidget = g_list_next(lwidget))
+    rebuild_children(GTK_ANYONE(lwidget->data));
 }
 
-static void show_user_callback(GtkWidget *child, zuser *target)
+static void set_tooltip(GtkAnyone *anyone, GtkWidget *child, zuser *user)
 {
-  zuser *user;
-  
-  /* Get the user from the child's user data. */
-  user = gtk_object_get_user_data(GTK_OBJECT(child));
-  
-  /* If it matches, show the child. */
-  if (user == target)
-    show_the_user(child, user);
-}
-
-static void show_the_user(GtkWidget *child, zuser *user)
-{
-  const ZLocations_t *loc;
-  int n;
   GString *tip;
-  GtkWidget *parent;
-  GtkAnyone *anyone;
+  int n;
+  const ZLocations_t *loc;
   
-  /* Get the parent of child.  This had better be a GtkAnyone... */
-  parent = child->parent; /* Abstraction, anyone? */
-  g_return_if_fail(parent != NULL);
-  anyone = GTK_ANYONE(parent);
-  
-  /* Show the widget. */
-  gtk_widget_show(child);
-
-  /* While we're at it, set a tooltip for the child widget. */
   tip = zuser_name(user);
   g_string_append(tip, "\n");
   
@@ -321,30 +360,3 @@ static void show_the_user(GtkWidget *child, zuser *user)
   g_string_free(tip, TRUE);
 }
 
-void gtk_anyone_hide_user(GtkAnyone *anyone, zuser *user)
-{
-  GList *base, *list;
-  
-  /* Use a callback to conditionally hide each user. */
-  gtk_container_foreach(GTK_CONTAINER(anyone),
-			(GtkCallback)hide_user_callback,
-			user);
-
-  /* Recurse into subgroups. */
-  base = anyone->children;
-  for (list = base; list; list = g_list_next(list))
-    gtk_anyone_hide_user(GTK_ANYONE(list->data), user);
-}
-
-static void hide_user_callback(GtkWidget *child, zuser *target)
-{
-  zuser *user;
-  
-  /* Get the user from the child's user data. */
-  user = gtk_object_get_user_data(GTK_OBJECT(child));
-  
-  /* If it matches, hide the child. */
-  if (user == target)
-    gtk_widget_hide(child);
-}
-
