@@ -1,7 +1,7 @@
 /*
  * gtkanyone.h: Show the user tree as a Gtk+ widget
  * David Maze <dmaze@mit.edu>
- * $Id: gtkanyone.h,v 1.11 2000/04/20 18:11:14 dmaze Exp $
+ * $Id: gtkanyone.h,v 1.12 2001/10/20 14:22:31 dmaze Exp $
  */
 
 #ifndef __GTKANYONE_H__
@@ -21,8 +21,6 @@ typedef struct _GtkAnyone GtkAnyone;
 struct _GtkAnyoneClass
 {
   GtkTreeClass parent_class;
-
-  GtkWidget *popup_menu;
 };
 
 struct _GtkAnyone
@@ -43,9 +41,6 @@ zgroup *gtk_anyone_group(const GtkAnyone *anyone);
 GString *gtk_anyone_get_filename(const GtkAnyone *anyone);
 void gtk_anyone_update(GtkAnyone *anyone, GtkProgress *progress);
 void gtk_anyone_update_user(GtkAnyone *anyone, zuser *user);
-void gtk_anyone_show_or_hide_user(GtkAnyone *anyone, zuser *user);
-void gtk_anyone_show_user(GtkAnyone *anyone, zuser *user);
-void gtk_anyone_hide_user(GtkAnyone *anyone, zuser *user);
 void gtk_anyone_construct(GtkAnyone *anyone, zgroup *group, GString *filename);
 
 #endif /* __GTKANYONE_H__ */
